package com.docus.server;

import com.docus.infrastructure.WebConfig;
import com.docus.infrastructure.web.json.JsonSerializerModule;
import com.docus.server.common.serializer.DefJsonSerializerModule;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.Charset;
import java.util.List;

@Configuration
public class DefaultWebConfig extends WebConfig {

    @Override
    public JsonSerializerModule jsonSerializerModu1e() {
        super.jsonSerializerModu1e();
        return new DefJsonSerializerModule();
    }

    //http请求工具 restTemplate
    @ConditionalOnMissingBean
    @Bean
    public RestTemplate restTemplate() {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setReadTimeout(3 * 60 * 1000);
        requestFactory.setConnectTimeout(3 * 60 * 1000);
        RestTemplate restTemplate = new RestTemplate(requestFactory);
        ObjectMapper objectMapper = new ObjectMapper();
        //]忽略未定义的属性
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        List<HttpMessageConverter<?>> messageConverters = restTemplate.getMessageConverters();
        messageConverters.removeIf(converter -> converter instanceof StringHttpMessageConverter);
        messageConverters.add(1, new StringHttpMessageConverter(Charset.forName("UTF-8")));
        HttpMessageConverter<?> jsonConverter = messageConverters.stream()
            .filter(p -> p instanceof MappingJackson2HttpMessageConverter).findFirst().orElse(null);
        if (jsonConverter != null) {
            ((MappingJackson2HttpMessageConverter) jsonConverter).setObjectMapper(objectMapper);
        }
        return restTemplate;
    }
}
