package com.docus.server.controller;

import com.docus.infrastructure.web.request.SearchDTO;
import com.docus.infrastructure.web.response.PageResult;
import com.docus.log.annotation.TrackLogGroup;
import com.docus.server.api.scheduling.management.SchCollectorApi;
import com.docus.server.common.process.CollectorVersionProcessor;
import com.docus.server.dto.scheduling.management.schcollector.AddSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.DeleteSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.EditSchCollectorDTO;
import com.docus.server.dto.scheduling.management.schcollector.UpdateSchCollectorDTO;
import com.docus.server.service.ISchCollectorService;
import com.docus.server.vo.scheduling.management.schcollector.SchCollectorVO;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 采集器管理 控制器类
 *
 * @author AutoGenerator
 * @since 2023-07-15
 */
@RestController
public class SchCollectorController implements SchCollectorApi {
    @Resource
    private ISchCollectorService iSchCollectorService;

    /**
     * 按主键查询
     *
     * @param id 主键Id
     * @return 实体
     */
    @Override
    public SchCollectorVO findById(String id) {
        return iSchCollectorService.findById(id);
    }

    /**
     * 关键字搜索
     *
     * @param searchDTO 搜索参数
     * @return 分页列表
     */
    @Override
    public PageResult<SchCollectorVO> search(SearchDTO searchDTO) {
        return iSchCollectorService.search(searchDTO);
    }

    /**
     * 新增
     *
     * @param addSchCollectorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean add(AddSchCollectorDTO addSchCollectorDTO) {
        return iSchCollectorService.add(addSchCollectorDTO);
    }

    /**
     * 编辑
     *
     * @param editSchCollectorDTO 编辑参数
     * @return 成功或失败
     */
    @Override
    public boolean edit(EditSchCollectorDTO editSchCollectorDTO) {
        return iSchCollectorService.edit(editSchCollectorDTO);
    }

    /**
     * 批量删除
     *
     * @param deleteSchCollectorDTO 删除参数
     * @return 成功或失败
     */
    @Override
    public int delete(DeleteSchCollectorDTO deleteSchCollectorDTO) {
        return iSchCollectorService.delete(deleteSchCollectorDTO);
    }

    /**
     * 选为当前版本并更新
     *
     * @return 成功或失败
     */
    @TrackLogGroup(group = "采集器管理", action = "更新", desc = "更新采集器版本", processor = CollectorVersionProcessor.class)
    @Override
    public boolean updateVersion(UpdateSchCollectorDTO updateDTO) {
        return iSchCollectorService.updateVersion(updateDTO);
    }
}
