package com.docus.server.service.impl;

import cn.hutool.core.util.StrUtil;
import com.docus.core.util.Func;
import com.docus.infrastructure.web.exception.ApiException;
import com.docus.infrastructure.web.exception.ExceptionCode;
import com.docus.server.service.IFileUploadService;
import com.docus.server.vo.scheduling.management.schcollectorversionfile.UploadFileVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.httpclient.util.URIUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.security.cert.CertificateException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Service
@Slf4j
public class FileUploadServiceImpl implements IFileUploadService {

    @Value("${file.uploadFolder:D://docus/}")
    private String saveFilePath;

    private static final String COLLECTOR_PACKAGES = "collector_packages";

    private static DateTimeFormatter ymdDtf = DateTimeFormatter.ofPattern("yyyyMMdd");

    private String userAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.139 Safari/537.36";

    @Override
    public List<UploadFileVO> uploadFile(MultipartFile[] multipartFiles, String pathKey) throws Exception {

        if (Func.isBlank(pathKey)) {
            pathKey = COLLECTOR_PACKAGES;
        }

        List<UploadFileVO> uploadFileVOList = new ArrayList<>();

        for (MultipartFile multipartFile : multipartFiles) {

            valid(multipartFile);

            uploadFileVOList.add(actionUploadFile(multipartFile, pathKey));
        }
        return uploadFileVOList;

    }

    private UploadFileVO actionUploadFile(MultipartFile multipartFile, String pathKey) throws Exception {

        String todayFormat = ymdDtf.format(LocalDateTime.now());

        String path = pathKey + "/" + todayFormat + "/" + UUID.randomUUID().toString() + "/";
        File fileDir = new File(saveFilePath + path);
        if (!fileDir.exists()) {
            fileDir.mkdirs();
        }
        String fileName = path + multipartFile.getOriginalFilename();

        File dest = new File(saveFilePath + fileName);

        multipartFile.transferTo(dest);


        UploadFileVO uploadFileVO = new UploadFileVO();
        uploadFileVO.setFileName(multipartFile.getOriginalFilename());
        uploadFileVO.setFileSize(multipartFile.getSize());
        uploadFileVO.setFileTitle(multipartFile.getName());
        uploadFileVO.setFileType(multipartFile.getContentType());
        uploadFileVO.setFilePath(fileName);

        return uploadFileVO;
    }

    @Override
    public void downloadFile(String path, HttpServletResponse response) {

        try {
            //获取要下载的文件
            File file = new File(saveFilePath + "/" + path);

            //设置响应的内容类型为二进制流，即文件类型
            response.setContentType("application/octet-stream");

            //设置文件名
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(file.getName(), "UTF-8"));

            //创建输入流
            FileInputStream inputStream = new FileInputStream(file);
            BufferedInputStream buffInputStream = new BufferedInputStream(inputStream);

            //创建输出流
            ServletOutputStream outputStream = response.getOutputStream();
            BufferedOutputStream buffOutputStream = new BufferedOutputStream(outputStream);

            //循环读取数据并写入到响应输出流中
            byte[] buffer = new byte[1024];
            int len = -1;
            while ((len = buffInputStream.read(buffer)) != -1) {
                buffOutputStream.write(buffer, 0, len);
            }

            //关闭流
            buffOutputStream.flush();
            buffOutputStream.close();
            outputStream.flush();
            outputStream.close();
            buffInputStream.close();
            inputStream.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 设置不验证主机
     */
    private final HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {
        @Override
        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };

    @Override
    public void downLoadFromUrl(String urlStr, String fileName, String savePath) throws Exception {
        long start = System.currentTimeMillis();
        urlStr = urlStr.replace("\\", "/");
        urlStr = URIUtil.encodePathQuery(urlStr);
        URL url = new URL(urlStr);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        try {
            boolean useHttps = urlStr.toLowerCase().startsWith("https");
            if (useHttps) {
                HttpsURLConnection https = (HttpsURLConnection) conn;
                trustAllHosts(https);
                https.setHostnameVerifier(DO_NOT_VERIFY);
            }
            //设置超时间为3秒
            //防止屏蔽程序抓取而返回403错误
            conn.setRequestProperty("User-Agent", userAgent);
            conn.setRequestProperty("Accept-Encoding", "identity");
            conn.setConnectTimeout(8 * 1000);
            conn.setReadTimeout(8 * 1000);
            conn = reload(conn);
            long length = conn.getContentLength();
            if (length < 0) {
                String values = conn.getHeaderField("Content-Length");
                if (values != null && !values.isEmpty()) {
                    length = Long.parseLong(values);
                }
            }
            InputStream inputStream = null;
            if (conn.getResponseCode() >= 400) {
                throw new Exception("文件不存在");
            } else {
                inputStream = conn.getInputStream();
            }
            //获取自己数组
            byte[] getData = readInputStream(inputStream);

            //文件保存位置
            File saveDir = new File(savePath);
            if (!saveDir.exists()) {
                saveDir.mkdir();
            }
            File file = new File(saveDir + File.separator + fileName);
            FileOutputStream fos = new FileOutputStream(file);
            fos.write(getData);
            if (fos != null) {
                fos.close();
            }
            if (inputStream != null) {
                inputStream.close();
            }
            long end = System.currentTimeMillis();
            log.info("info:" + url + " download success;用时:" + (end - start) + "ms");
        } catch (Exception ex) {
            throw ex;
        } finally {
            // 断开连接，释放资源
            conn.disconnect();
        }
    }

    /**
     * 处理多级302等跳转
     *
     * @param uc
     * @return
     * @throws Exception
     */
    private HttpURLConnection reload(HttpURLConnection uc) throws Exception {

        HttpURLConnection huc = uc;

        if (huc.getResponseCode() == HttpURLConnection.HTTP_MOVED_TEMP
            || huc.getResponseCode() == HttpURLConnection.HTTP_MOVED_PERM) {// 302, 301
            String url = huc.getHeaderField("Location");
            url = url.replace("\\", "/");
            return reload((HttpURLConnection) new URL(url).openConnection());
        }
        return uc;
    }

    /**
     * 覆盖java默认的证书验证
     */
    private final TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
        @Override
        public void checkClientTrusted(java.security.cert.X509Certificate[] x509Certificates, String s) throws CertificateException {

        }

        @Override
        public void checkServerTrusted(java.security.cert.X509Certificate[] x509Certificates, String s) throws CertificateException {

        }

        @Override
        public java.security.cert.X509Certificate[] getAcceptedIssuers() {
            return new java.security.cert.X509Certificate[]{};
        }
    }};

    /**
     * 信任所有
     *
     * @param connection
     * @return
     */
    private SSLSocketFactory trustAllHosts(HttpsURLConnection connection) {
        SSLSocketFactory oldFactory = connection.getSSLSocketFactory();
        try {
            SSLContext sc = SSLContext.getInstance("TLS");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            SSLSocketFactory newFactory = sc.getSocketFactory();
            connection.setSSLSocketFactory(newFactory);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return oldFactory;
    }

    /**
     * 从输入流中获取字节数组
     *
     * @param inputStream
     * @return
     * @throws IOException
     */
    public byte[] readInputStream(InputStream inputStream) throws IOException {
        byte[] buffer = new byte[1024];
        int len = 0;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        while ((len = inputStream.read(buffer)) != -1) {
            bos.write(buffer, 0, len);
        }
        bos.close();
        return bos.toByteArray();
    }


    private void valid(MultipartFile multipartFile) {
        if (multipartFile.isEmpty()) {
            throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "上传失败，请选择文件！");
        }

        String originalFilename = multipartFile.getOriginalFilename();
        if (StrUtil.isBlank(originalFilename)) {
            throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "未找到原文件文件名！");
        }
        if (!originalFilename.contains(".")) {
            throw new ApiException(ExceptionCode.ParamIllegal.getCode(), "原文件名未识别到后缀！");
        }
    }

}
