package com.docus.server.common.netty.client;

import com.docus.server.common.netty.client.handler.ClientHandler;
import com.docus.server.common.netty.client.handler.HeartbeatHandler;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.handler.timeout.IdleStateHandler;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;


@AllArgsConstructor
@NoArgsConstructor
public class NettyClientInitializer extends ChannelInitializer<Channel> {

    private NettyClientProperties nettyProperties;

    private NettyClient nettyClient;

    private String appName;

    @Override
    protected void initChannel(Channel channel) throws Exception {
        channel.pipeline()
            // 加载空闲监听器
            .addLast(new IdleStateHandler(nettyProperties.getReaderIdleTimeSeconds(),
                nettyProperties.getWriterIdleTimeSeconds(), nettyProperties.getAllIdleTimeSeconds()))
            // 加载加码解码处理器，同时解决粘包拆包问题
//            .addLast(new ProtobufVarint32FrameDecoder())
//            .addLast(new ProtobufDecoder(Payload.Message.getDefaultInstance()))
//            .addLast(new ProtobufVarint32LengthFieldPrepender())
//            .addLast(new ProtobufEncoder())
            // 加载心跳处理器
            .addLast(new HeartbeatHandler(nettyClient, nettyProperties, appName))
            // 加载业务处理器
            .addLast(new ClientHandler())
            .addLast();
    }


}
