package com.docus.server.common.netty.server;

import io.netty.channel.Channel;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 客户端IP和通信信道的映射
 */
@Component
@Slf4j
public class ChannelRepository {

    /**
     * <collectorId-Channel>
     */
    private final static Map<String, Channel> COLLECTOR_ID_CHANNEL_CACHE_MAP = new ConcurrentHashMap<>();

    /**
     * 终端上线
     */
    public void put(String collectorId, Channel channel) {

        //缓存
        COLLECTOR_ID_CHANNEL_CACHE_MAP.put(collectorId, channel);

        AttributeKey<String> attributeKey = AttributeKey.valueOf("ip");
        channel.attr(attributeKey).set(collectorId);

    }

    public String getClientKey(Channel channel) {

        AttributeKey<String> key = AttributeKey.valueOf("ip");

        if (channel.hasAttr(key)) {
            return channel.attr(key).get();
        }
        return null;
    }

    public Channel get(String key) {
        return COLLECTOR_ID_CHANNEL_CACHE_MAP.get(key);
    }

    /**
     * 终端离线
     */
    public void remove(String key) {
        COLLECTOR_ID_CHANNEL_CACHE_MAP.remove(key);
    }

}
