package com.docus.server.collect.basic.http;

import com.docus.log.annotation.TrackRetryListener;
import com.docus.server.collect.ICollector;
import com.docus.server.collect.web.job.AbstractCollectJob;
import com.docus.server.entity.TaskConfig;
import com.docus.server.record.common.pojo.dto.TBasicDTO;
import com.docus.server.util.PeriodTime;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 主动http请求基础数据job
 *
 * @see AbstractCollectJob
 */
@Component
public class TBasicHttpCollectJob extends AbstractCollectJob<TBasicDTO> {

    @Resource(name = "httpTBasicCollectorImpl")
    private ICollector collector;

    /**
     * xxl jobhandler 手工执行
     */
    @XxlJob("startAllTBasicHttpCollect")
    @TrackRetryListener("startAllTBasicHttpCollect")
    public void startAllTBasicHttpCollect(String param) {
        super.startCollectAll(getJobParam(param));
    }

    /**
     * xxl jobhandler 配置定时启动，如果未完成任务跳过。
     */
    @XxlJob("startIncTBasicHttpCollect")
    @TrackRetryListener("startIncTBasicHttpCollect")
    public void startIncTBasicHttpCollect(String param) {
        super.startCollectIncrement(getJobParam(param));
    }


    @Override
    public void doHandle(List<TBasicDTO> results, String param) {
        basicService.batchSaveBasics(results);
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<TBasicDTO> execute(PeriodTime periodTime, TaskConfig taskConfig) {
        //需要根据不同医院去解析不同的内容。使用接口实现的方式。
        return collector.page(periodTime, taskConfig);
    }
}
